/******************************************************************************

 bvs_strom.c version 1.0

  - program na razeni udaju pomoci binarniho vyhledavaciho stromu

 Copyright (c) 2001 Petr Kratky <pkratky@student.zcu.cz>

/********************************* Include ***********************************/

#include<stdio.h>
#include<stdlib.h>

/******************************** Constants **********************************/

#define DELKA_JMENA 20
#define DELKA_DRUZST 11
#define VEL_TAB 25
#define VEL_RADKY 50

/********************************** Types ************************************/

typedef struct polozka { /* Obsahuje udaje o jednotlivych zavodnicich */
  int cislo;             /* Startovni cislo */
  char jmeno[DELKA_JMENA];        /* Jmeno */
  char druzstvo[DELKA_DRUZST];     /* Druzstvo do ktereho patri */
  int body;              /* Pocet ziskanych bodu */
} POLOZKA;

typedef struct uzel {            /* Uzel stromu */
  int body;                      /* Hodnota podle, ktere se tridi - poc. zisk. bodu */
  int cislo;                     /* Cislo do tabulky */
  struct uzel *p_left, *p_right; /* Levy a pravy syn uzlu */
} UZEL;

/****************************** Global variables ******************************/

POLOZKA tab[VEL_TAB];  /* Pole lidi ktere chceme tridit */
int pocet = 0;  /* Pocet nactenych polozek ze souboru */
UZEL *p_root = NULL;  /* Koren stromu, NULL - prazdny strom */

/*************************** Function definitions *****************************/

/* Zaradi cislo x do tabulky */
void zarad(int x,int cislo_tab)
{
  UZEL *p_pom,*p_prvek;  /* Pomocne pointery */

  if ((p_prvek = (UZEL *) malloc(sizeof(UZEL))) == NULL) { /* Alokuje pamet pro uzel */
    printf("Nedostatek pameti !\n");
    exit(1);
  }
  /* Prirazeni na pomocny pointer */
  p_prvek->body = x;
  p_prvek->cislo = cislo_tab;
  p_prvek->p_left = NULL;
  p_prvek->p_right = NULL;

  if (p_root==NULL) /* Jestlize je strom prazdny, prvek rovnou zaradime */
    p_root=p_prvek;
  else {
    p_pom = p_root;   /* Uchovani korene */
    while (p_pom != NULL) {
      if (x < p_pom->body) { /* Test zda novy prvek je mensi nez stavajici */
	      if (p_pom->p_left == NULL) {
      	  p_pom->p_left = p_prvek;  /* Zarazeni prvku do leva*/
	        break;
	      }
       	else p_pom = p_pom->p_left;
      }
      else {
	      if (p_pom->p_right == NULL) {
	        p_pom->p_right = p_prvek;  /* Zarazeni prvku do prava*/
	        break;
	      }
        else p_pom=p_pom->p_right;
      }
    }
  }
}
/*----------------------------------------------------------------------------*/

/* Vypise serazenou vysledkoovou listinu */
void vypis_vysl(UZEL *p_odkud)
{
  static int poc = 0; /* Pocet dosud vypsanych zavodniku */

  /* Vypis od nejvetsiho po nejmensi */
  if (p_odkud->p_right != NULL)
    vypis_vysl(p_odkud->p_right);
  printf("%2d %-20s %s %d", p_odkud->cislo+1, tab[p_odkud->cislo].jmeno,
      	 tab[p_odkud->cislo].druzstvo, p_odkud->body);
  poc++; /* Pocitadlo poctu vypsanych udaju */
  if (poc < VEL_TAB)  /* Test zda jsme na konci */
    putchar('\n');
  if (p_odkud->p_left != NULL)
    vypis_vysl(p_odkud->p_left);
}
/*----------------------------------------------------------------------------*/

/* Zrusi strom a uvolni pamet */
void zrus(UZEL *p_odkud)
{
  if (p_odkud->p_left != NULL) zrus(p_odkud->p_left);
  free((void *) p_odkud);
  if (p_odkud->p_right != NULL) zrus(p_odkud->p_right);
}
/*----------------------------------------------------------------------------*/

/* Otevre soubor a nacte data do prislusne struktury */
void nacti(char *jmeno,int max)
{
  FILE *fr;              /* Ukazatel na soubor */
  int i;                 /* Pomocna promenna */
  char radka[VEL_RADKY];  /* Velikost radky */

  if ((fr = fopen(jmeno,"r")) == NULL) {
    printf("Nelze otevrit soubor %s !\n",jmeno);
    exit(1);
  }
  i = 0;
  /* Nacteni radky a rozdeleni do jednotlivych polozek */
  while (fgets(radka,VEL_RADKY,fr) != NULL && i<max) {
    sscanf(radka,"%d %s %s %d",&tab[i].cislo, tab[i].jmeno, tab[i].druzstvo,
    &tab[i].body);
    i++;
  }
  pocet = i;
  fclose(fr);  /* Uzavreni souboru */
}

/*********************************** Main ************************************/

main()
{
  int k;  /* Pomocna promenna */

  nacti("zavod.dat", VEL_TAB);
  for (k = 0; k < pocet; k++)
    zarad(tab[k].body, k); /* Vytvori strom */

  if (p_root != NULL) {
    vypis_vysl(p_root);    /* Vypise strom */
    getchar();
    zrus(p_root);          /* Zrusi strom */
    p_root = NULL;
  }
  else printf("Strom je prazdny !\n");
}
/******************************** End of file *********************************/
